package crypt.jce;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.crypto.SecretKey;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.BevelBorder;

public class BasicCryptGUI extends JPanel implements ActionListener, MouseListener {

	//layered Pane
	JLayeredPane layers;

	FilePanel[] sources;
	KeyPanel[] keys;
	FilePanel cryptInput;
	FilePanel cryptOutput;
	KeyPanel keyInput;
	JComboBox cipherList;
	JTextArea outputText;
	JButton encryptButton;
	JLabel stats;

	public BasicCryptGUI(int width,int height) {
		/* Much of the following may be handled differently later, either in
		   the constructor or with an element of randomness */
		
		String[] phrase = new String[4];
		phrase[0] = "The quick brown";
		phrase[1] = "fox jumps over";
		phrase[2] = "the lazy dog";
		phrase[3] = "while eating cheese";

		/* make a layered pane..  */
		layers = new JLayeredPane();
		layers.setPreferredSize(new Dimension(width, height));

		//layout managers are evil and must be burned :)
		//setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		/* add layers to this panel */
		add(layers, BorderLayout.CENTER);

		/* create the scratchpad panel and place it on the layered pane */
		JPanel scratch = new JPanel();
		scratch.setName("Scratch");
		scratch.addMouseListener(this);

		//layout managers are evil and must be burned :)
		//scratch.setLayout(new GridBagLayout());
		scratch.setBounds(0, 0, width, height / 2);
		layers.add(scratch, JLayeredPane.DEFAULT_LAYER);

		/* Controlbox area */
		JPanel controlBox = new JPanel();
		controlBox.setBounds(0, (height / 2) + 1, width, 50);
		controlBox.setBackground(Color.darkGray);
		layers.add(controlBox, JLayeredPane.DEFAULT_LAYER);

		/* Output Area */
		JPanel outputArea = new JPanel();
		JPanel rightArea = new JPanel();
		rightArea.setLayout(new BorderLayout());
		rightArea.setBounds(80, (height / 2)+52, width - 80, 150);

		outputArea.setLayout(new BorderLayout());
		// Dimensions used to be 0,250,width,150
		outputArea.setBounds(0, (height/3)*2, width - 10, 130);
		// Handle text box
		outputText = new JTextArea(7, height / 8);
		outputText.setLineWrap(true);
		outputText.setWrapStyleWord(true);
		//outputText.setBounds(height/4,width/32,width*27/32,5);
		outputText.setEditable(false);
		JScrollPane areaScrollPane = new JScrollPane(outputText);
		areaScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		areaScrollPane.setPreferredSize(new Dimension(width * 27 / 32, height * 10 / 40));
		areaScrollPane.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));

		rightArea.add(areaScrollPane, BorderLayout.NORTH);

		stats = new JLabel();
		stats.setBounds(0, 0, width, 30);
		rightArea.add(stats, BorderLayout.SOUTH);

		outputArea.add(rightArea, BorderLayout.EAST);

		layers.add(outputArea, JLayeredPane.DEFAULT_LAYER); // [insert output area here]

		sources = new FilePanel[10];
		keys = new KeyPanel[10];
		int x = 20, y = 20;
		for (int i = 0; i < 4; i++) {
			sources[i] = new FilePanel("File " + new Integer(i).toString(), "10110101", phrase[i]);
			sources[i].setParent(this);
			sources[i].setBounds(x, y, 60, 36);
			x += 64;
			layers.add(sources[i], JLayeredPane.PALETTE_LAYER);
		}
		x = 20;
		y = 60;
		for (int i = 0; i < 4; i++) {
			keys[i] = new KeyPanel("Key" + new Integer(i).toString(), "ACYHXNT");
			keys[i].setParent(this);
			keys[i].setBounds(x, y, 60, 36);
			x += 64;
			layers.add(keys[i], JLayeredPane.PALETTE_LAYER);
		}

		cryptOutput = new FilePanel("Output", "Not Set ");
		cryptOutput.setBounds(20, (height/4)*3, 60, 40);
		//This should probably go in the constructor, but that'll come later
		//The same is true for all the places I'm about to put it below.
		cryptOutput.setParent(this);
		cryptOutput.setMobile(false);
		layers.add(cryptOutput, JLayeredPane.PALETTE_LAYER);

		int buttonwidth = 80;

		JButton addFileButton = new JButton("Add File");
		addFileButton.setName("AddFile");
		addFileButton.setActionCommand("addfile");
		addFileButton.addActionListener(this);
		addFileButton.setBounds(width - 20 - buttonwidth, 20, buttonwidth, 28);
		layers.add(addFileButton, JLayeredPane.PALETTE_LAYER);

		JButton addKeyButton = new JButton("Add Key");
		addKeyButton.setName("AddKey");
		addKeyButton.setActionCommand("addkey");
		addKeyButton.addActionListener(this);
		addKeyButton.setBounds(width - 20 - buttonwidth, 58, buttonwidth, 28);
		layers.add(addKeyButton, JLayeredPane.PALETTE_LAYER);

		// Why Java uses HTML in something like this, I don't know, but it's kinda nice in
		// a twisted sense...
		JButton tidyButton = new JButton("<html><center>Tidy<br>Scratch</center></html>");
		tidyButton.setName("Tidy");
		tidyButton.setActionCommand("tidy");
		tidyButton.addActionListener(this);
		tidyButton.setBounds(width - 20 - buttonwidth, 58 + 28 + 10, buttonwidth, 40);
		layers.add(tidyButton, JLayeredPane.PALETTE_LAYER);

		String[] cipherStrings = CryptUtils.getSymmetricCiphers();
		cipherList = new JComboBox(cipherStrings);

		cipherList.addItem("Vigenere");
		cipherList.setSelectedIndex(0);

		cipherList.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Someone chose a new cipher
			}
		});
		cryptInput = new FilePanel("Input", "Not Set ");
		cryptInput.setParent(this);
		cryptInput.setMobile(false);
		controlBox.add(cryptInput); // [insert input drag box here]
		controlBox.add(cipherList);
		keyInput = new KeyPanel("Key", "Not Set");
		keyInput.setActive(false);
		keyInput.setParent(this);
		keyInput.setMobile(false);
		controlBox.add(keyInput); // [insert key drag box here]
		encryptButton = new JButton("Encrypt");
		encryptButton.setName("Encrypt");
		encryptButton.setActionCommand("encrypt");
		encryptButton.addActionListener(this);
		controlBox.add(encryptButton);
	}

	public JButton getEncryptButton() {
		return encryptButton;
	}

	public BasicCryptGUI getCryptGUI() {
		return this;
	}

	public JLayeredPane getLayers() {
		return layers;
	}

	public void setCryptOutput(FilePanel out) {
		cryptOutput = out;
	}

	public void setCryptInput(FilePanel out) {
		cryptInput = out;
	}

	public FilePanel getCryptInput() {
		return cryptInput;
	}

	public FilePanel getCryptOutput() {
		return cryptOutput;
	}

	public KeyPanel getKeyInput() {
		return keyInput;
	}

	public void mousePressed(MouseEvent e) {
		System.out.println(e.getComponent().getName());
	}

	public void mouseReleased(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mouseClicked(MouseEvent e) {
		if (e.getClickCount() == 2) {
			// insert doubleclick code here
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("encrypt")) {
			boolean hassecret = true;
			boolean encrypting = true;
			System.out.println("En/Decrypt pressed!");

			if (cryptInput.getBlobFile().getText().equals("")) {
				System.out.println("No input set");
				return;
			}

			if (keyInput.getKey() == null) {
				System.out.println("No key set");
				return;
			}

			int i = cipherList.getSelectedIndex();
			SecretKey k = keyInput.getKey().getKey(i);
			BlobFile b = cryptInput.getBlobFile();
			String alg = cipherList.getSelectedItem().toString();
			String phr = keyInput.getKey().getPhrase();
			Cryptographer myCrypt;
			System.out.println(alg);

			if (k != null) {
				myCrypt = new Cryptographer(alg, k);
			} else {
				myCrypt = new Cryptographer(alg, phr);
			}

			long start = System.currentTimeMillis();
			String result = "";
			String original = b.getText();
			// Do it 100 times to get a sensible time
			for (int ij = 0; ij < 100; ij++) {
				if (encryptButton.getText().equals("Encrypt")) {
					result = myCrypt.encrypt(original);
					encrypting = true;
				} else {
					result = myCrypt.decrypt(original);
					encrypting = false;
				}
			}
			long end = System.currentTimeMillis();

			String statInfo = "Time taken (x100): " + new Long(end - start).toString() + " milliseconds     ";
			statInfo = statInfo + "Expansion factor: " + new Double((double) result.length() / (double) original.length()).toString();
			statInfo = statInfo + "";
			statInfo = statInfo + "";
			statInfo = statInfo + "";
			stats.setText(statInfo);
			outputText.setText(result);

			String title;
			if (encrypting == true) {
				title = "*" + cryptInput.getTop().getText();
			} else {
				title = cryptInput.getTop().getText();
				if (title.substring(0, 1).equals("*")) {
					title = title.substring(1, title.length());
				}
			}
			cryptOutput.rewrite(title, "01001010");

			cryptOutput.setToolTipText(result);
			cryptOutput.setBlobFile(new BlobFile(result, true));
			cryptOutput.setBounds(cryptOutput.getX(), cryptOutput.getY(), 60, 36);
			cryptOutput.setMobile(true);
			cryptOutput.paintImmediately(0,0,cryptOutput.getWidth(), cryptOutput.getHeight());

		}
	}
}
