package crypt.jce;

import java.security.Provider;
import java.security.Security;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Contains static methods primarily used to interface with the JCE
 */

class CryptUtils
{
	private static String[] symmetricAlgorithms = null;

	/**
	 * Iterates through available KeyGenerators to generate a list of
	 * avalable symmetric algorithms that have KeyGenerators
	 *
	 * @return	an array containing the names of available algorithms
	 */
	public static String[] getSymmetricAlgorithms()
	{
		/* We keep a cached copy of this data to reuse after first run
		 * as there are lots of providers to iterate through and we're
		 * only interested in a few of them */
		if( symmetricAlgorithms != null )
			return symmetricAlgorithms;

		Set result = new HashSet();

		/* Get all available providers */
		Provider[] providers = Security.getProviders();
		for (int i = 0; i < providers.length; i++)
		{
			/* Iterate over services provided by each provider */
			Set keys = providers[i].keySet();
			for (Iterator it = keys.iterator(); it.hasNext(); )
			{
				String key = ((String)it.next()).split(" ")[0];

				if (key.startsWith("KeyGenerator."))
					result.add(key.substring(13));
				else
				{
					// Return aliases too
					if (key.startsWith("Alg.Alias.KeyGenerator."))
						result.add(key.substring(23));
				}
			}
		}
		/* Cache result */
		symmetricAlgorithms = (String[]) result.toArray(new String[result.size()]);
		return symmetricAlgorithms;
	}

	/**
	 * Iterates through available KeyGenerators and then discards all
	 * that have names beginning with Hmac, producing a list of
	 * available symmetric ciphers which have KeyGenerators
	 *
	 * @return	an array containing the names of available ciphers
	 */
	public static String[] getSymmetricCiphers()
	{
		String[] algorithms = getSymmetricAlgorithms();
		ArrayList ciphers = new ArrayList();

		for (int i = 0; i < algorithms.length; i++)
			if (!algorithms[i].startsWith("Hmac"))
				ciphers.add(algorithms[i]);

		return (String[]) ciphers.toArray(new String[ciphers.size()]);
	}

}
