package crypt.jce;

import java.io.UnsupportedEncodingException;

import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.SecretKey;
import crypt.common.Base64;

public class Cryptographer {
	Cipher ecipher, dcipher;
	String algorithm;
	SecretKey key;
	String phrase;

	Cryptographer(String alg, SecretKey ky) {
		try {
			ecipher = Cipher.getInstance(alg);
			dcipher = Cipher.getInstance(alg);
			ecipher.init(Cipher.ENCRYPT_MODE, ky);
			dcipher.init(Cipher.DECRYPT_MODE, ky);
			setAlgorithm(alg);
			setKey(ky);
			setPhrase("");
		} catch (javax.crypto.NoSuchPaddingException e) {
		} catch (java.security.NoSuchAlgorithmException e) {
		} catch (java.security.InvalidKeyException e) {
		}
	}

	Cryptographer(String alg, String phr) {
		setAlgorithm(alg);
		setPhrase(phr);
		setKey(null);
	}

	public void setPhrase(String phr) {
		phrase = phr;
	}

	public String getPhrase() {
		return phrase;
	}

	public void setAlgorithm(String alg)
	{
		algorithm = new String(alg);
	}

	public void setKey(SecretKey ky)
	{
		key = ky;
	}

	public String getAlgorithm()
	{
		return algorithm;
	}

	public SecretKey getKey()
	{
		return key;
	}

	public String encrypt(String str) {

		if (this.getKey() == null && this.getAlgorithm().equalsIgnoreCase("vigenere") == true) {
			return vigenereEncrypt(str,this.getPhrase());
		}
		
		if (this.getKey() == null)
			return "";

		try {

			// Encode the string into bytes using utf-8
			byte[] utf8 = str.getBytes("UTF8");

			// Encrypt
			byte[] enc = ecipher.doFinal(utf8);

			// Encode bytes to base64 to get a string
			return Base64.encode(enc);

		} catch (javax.crypto.BadPaddingException e) {
		} catch (IllegalBlockSizeException e) {
		} catch (UnsupportedEncodingException e) {
//		} catch (java.io.IOException e) {
		}
		return null;
	}

	public String decrypt(String str) {
		if (this.getKey() == null && this.getAlgorithm().equalsIgnoreCase("vigenere") == true) {
			return vigenereDecrypt(str,this.getPhrase());
		}
		
		if (this.getKey() == null)
			return "";

		try {
			// Decode base64 to get bytes
			byte[] dec = Base64.decode(str.getBytes())/*new sun.misc.BASE64Decoder().decodeBuffer(str)*/;

			// Decrypt
			byte[] utf8 = dcipher.doFinal(dec);

			// Decode using utf-8
			return new String(utf8, "UTF8");
		} catch (javax.crypto.BadPaddingException e) {
		} catch (IllegalBlockSizeException e) {
		} catch (UnsupportedEncodingException e) {
//		} catch (java.io.IOException e) {
		}
		return null;
	}


	private String vigenereEncrypt(String str, String phr) {
		char[] ch = str.toCharArray();
		char[] ph = phr.toCharArray();
		String result = "";
		int j = 0;
		for (int i = 0 ; i < ch.length ; i++)
		{
			int t = (int) ch[i] + ( (int) ph[j] - 65);

			result = result + new Character((char) t).toString();

			j++;
			if (j >= ph.length)
				j = 0;
		}

		return result;
	}

	private String vigenereDecrypt(String str, String phr) {
		char[] ch = str.toCharArray();
		char[] ph = phr.toCharArray();
		String result = "";
		int j = 0;
		for (int i = 0 ; i < ch.length ; i++)
		{
			int t = (int) ch[i] - ((int) ph[j] - 65);

			result = result + new Character((char) t).toString();

			j++;
			if (j >= ph.length)
				j = 0;
		}

		return result;
	}



}
