package crypt.jce;

/*
MobilePanel.java
Abstract Class containing common code related to mobile panels.
*/

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;

public abstract class MobilePanel extends JPanel implements MouseMotionListener, MouseListener {

	private int offsetx;
	private int offsety;
	// Probably shouldn't be public - accessor should be added later
	public int startx,starty;
	private int x0,x1,y0,y1;
	private boolean mobile=true;
	private JLabel top;
	private JLabel bottom;
	// A link back to the parent GUI
	private BasicCryptGUI parent;

	public MobilePanel (String label, String desc) {
		//setBlobFile (new BlobFile());
		setBounds(32,32,32,32);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		top = new JLabel(label);
		bottom = new JLabel(desc);
		add(top);
		add(bottom);
		addMouseMotionListener(this);
		addMouseListener(this);
		this.setName(label);
	}
    
    public MobilePanel(String imagefile) {
		setBounds(32,32,32,32);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		ImageIcon img = new ImageIcon(imagefile);
		top = new JLabel(img);
		bottom = new JLabel("");
		add(top);
		add(bottom);
		addMouseMotionListener(this);
		addMouseListener(this);
		this.setName(imagefile);    
    }

	public void setParent(BasicCryptGUI p) {
		parent = p;
	}

	//This is very much like the built in getParent(), except that it
	//always returns a CryptGUI rather than a Container, and so just makes
	//life that little nicer in my head. Yes, it could be replaced with
	//the conventional getParent() later :)
	public BasicCryptGUI getGUIParent() {
		return parent;
	}

	public void rewrite(String t, String b) {
		remove(getBottom());
		remove(getTop());
		JLabel newTop = new JLabel(t);
		JLabel newBottom = new JLabel(b);
		add(newTop);
		add(newBottom);
		setTop(newTop);
		setBottom(newBottom);
		revalidate();
	}

	public JLabel getBottom() {
		return bottom;
	}

	public void setBottom(JLabel t) {
		bottom = t;
	}

	public JLabel getTop() {
		return top;
	}

	public void setTop(JLabel t) {
		top = t;
	}

	public void sweetSpot() {
		//called when mouse is released over special area
		//override this if needed
	}

	public void notSweetSpot() {
		//called when mouse is released and misses the special area
		//override this if needed
	}

	public void otherSpot(int x, int y) {
		//called after the speetspot checks have been made, so we
		//could in theory catch as many other spots manually as we like.
		//Much of this could be done in notSweetSpot, but this may be
		//more convenient in some instances as it's called regardless of
		//whether the sweetspot is hit or not.
	}

	public void setSweetSpot(int X0,int Y0,int X1,int Y1) {
		x0=X0;y0=Y0;x1=X1;y1=Y1;
	}

	protected void setMobile(boolean move) {
		mobile=move;
	}

	public void mousePressed(MouseEvent e) {
		startx = getX();
		starty = getY();
		offsetx = e.getX();
		offsety = e.getY();
	}

	public void mouseReleased(MouseEvent e) {
		otherSpot(e.getX()+getX(),e.getY()+getY());
		if(getX()+e.getX()>=x0 && getX()+e.getX()<=x1 && getY()+e.getY()>=y0 && getY()+e.getY()<=y1)
			sweetSpot();
		else
			notSweetSpot();
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mouseClicked(MouseEvent e) {
		if (e.getClickCount() == 2)
		{
			System.out.println("Double clicked on "+this.getName());
		}
	}

	public void mouseMoved(MouseEvent e) {
	}

	public void mouseDragged(MouseEvent e) {
		if(mobile)
			setLocation(getX()-offsetx+e.getX(),getY()-offsety+e.getY());
	}

}
